import pickle
import multiprocessing
import numpy as np
import loopy as lp
import pyopencl as cl
from meshmode.array_context import PyOpenCLArrayContext
from sumpy.visualization import FieldPlotter

cl_ctx = cl.create_some_context()
queue = cl.CommandQueue(cl_ctx)
actx = PyOpenCLArrayContext(queue)

with open("kernel.pkl", "rb") as fp:
    knl = pickle.load(fp)

fplot = FieldPlotter(np.zeros(2), extent=3, npoints=1000)
targets = actx.from_numpy(fplot.points)
ntargets = targets[0].shape[0]

k_t = np.complex128(116.8 + 116.8j)
k_p = np.complex128(1 + 3.4e-5j)
I = np.complex128(1j)


def run_knl(knl):
    evt, res = knl(
        queue, target_points=targets, n_targets=ntargets,
        result=np.zeros(ntargets, dtype=np.complex128),
        k_p=k_p, k_t=k_t, I=I)
    res = res["result"]
    return res


ncpus = multiprocessing.cpu_count()
tknl = lp.split_iname(knl,
                      split_iname="itgt",
                      inner_length=ncpus//2,
                      inner_tag="g.0")

res = run_knl(knl)

for i in range(10):
    tagged_res = run_knl(tknl)
    diff = np.abs(res - tagged_res)
    print("number of bad points =", np.sum(diff>1e-12),
          "\tmax diff =", np.max(diff))

# print(knl)
# print(tknl)

if 0:
    fplot.write_vtk_file("issue.vts", [
        ("untagged", res),
        ("tagged", tagged_res),
        ("diff", diff),
    ])
